@@ -16,68 +16,34 @@
         modified by SRC, incorporated herein by reference.
          
 	**********************
+	
+	v2.00 (95/09/06)
+	  - THIS IS ONLY A SUMMARY.  The complete changelog is available
+	    from me upon request.
 
-	v1.91 ALPHA (95/07/02)
-	  - Oops.  Exception packets hit us again!  I remembered to test
-	    them in Windows-protocol mode, but due to the many various
-	    changes they broke in RFC1201 instead.  All fixed.
-	  - A long-standing bug with "exception" packets not setting
-	    protocol_id properly has been corrected.  This would have caused
-	    random problems talking to non-Linux servers.  I've also sent in
-	    a patch to fix this in the latest stable ARCnet (now 1.02).
-	  - ARC_P_IPX is an RFC1201 protocol too.  Thanks, Tomasz.
-	  - We're now "properly" (I think) handling the multiple 'tbusy' and
-	    'start' flags (one for each protocol device) better.
-	  - The driver should now start without a NULL-pointer dereference
-	    if you aren't connected to the network.
+	  - ARCnet RECON messages are now detected and logged.  These occur
+	    when a new computer is powered up on the network, or in a
+	    constant stream when the network cable is broken.  Thanks to
+	    Tomasz Motylewski for this.  You must have D_EXTRA enabled
+	    if you want these messages sent to syslog, otherwise they will
+	    only show up in the network statistics (/proc/net/dev).
+	  - The TX Acknowledge flag is now checked, and a log message is sent
+	    if a completed transmit is not ACK'd.  (I have yet to have this
+	    happen to me.)
+	  - Debug levels are now completely different.  See the README.
+	  - Many code cleanups, with several no-longer-necessary and some
+	    completely useless options removed.
+	  - Multiprotocol support.  You can now use the "arc0e" device to
+	    send "Ethernet-Encapsulation" packets, which are compatible with
+	    Windows for Workgroups and LAN Manager, and possibly other
+	    software.  See the README for more information.
+	  - Documentation updates and improvements.
 	  
-	v1.90 ALPHA (95/06/18)
-	  - Removal of some outdated and messy config options (no one has
-	    ever complained about the defaults since they were introduced):
-	    DANGER_PROBE, EXTRA_DELAYS, IRQ_XMIT, CAREFUL_XMIT,
-	    STRICT_MEM_DETECT, LIMIT_MTU, USE_TIMER_HANDLER.  Also took out
-	    a few "#if 0" sections which are no longer useful.
-	  - Cleaned up debug levels - now instead of levels, there are
-	    individual flags.  Watch out when changing with ifconfig.
-	  - More cleanups and beautification.  Removed more dead code and
-	    made sure every function was commented.
-	  - Fixed the DETECT_RECONFIGS option so that it actually _won't_
-	    detect reconfigs.  Previously, the RECON irq would be disabled
-	    but the recon messages would still be logged on the next normal
-	    IRQ.
-	  - Initial support for "multiprotocol" ARCnet (this involved a LOT
-	    of reorganizing!).  Added an arc0w device, which allows us to
-	    talk to "Windows" ARCnet TCP/IP protocol.  To use it, ifconfig
-	    arc0 and arc0w (in that order).  For now, Windows-protocol
-	    hosts should have routes through arc0w - eventually I hope to
-	    make things more automatic.
-	v1.11 ALPHA (95/06/07)
-	  - Tomasz saves the day again with patches to fix operation if the
-	    new VERIFY_ACK option is disabled.
-	  - LOTS of little code cleanups/improvements by Tomasz.
-	  - Changed autoprobe, since the "never-changing command port"
-	    probe was causing problems for some people.  I also reset the
-	    card fewer times during the probe if DANGER_PROBE is defined,
-	    since DANGER_PROBE seems to be a more reliable method anyway.
-	  - It looks like the null-pointer problem was finally REALLY fixed
-	    by some change from Linux 1.2.8 to 1.2.9.  How handy!
-	v1.10 ALPHA (95/04/15)
-	  - Fixed (?) some null-pointer dereference bugs
-	  - Added better network error detection (from Tomasz) - in
-	    particular, we now notice when our network isn't connected,
-	    also known as a "network reconfiguration."
-	  - We now increment lp->stats.tx_dropped in several more places,
-	    on a suggestion from Tomasz.
-	  - Minor cleanups/spelling fixes.
-	  - We now monitor the TXACK bit in the status register: we don't do
-	    anything with it yet, just notice when a transmitted packet isn't
-	    acknowledged.
-	  - Minor fix with sequence numbers (sometimes they were being sent in
-	    the wrong order due to Linux's packet queuing).
 	v1.01 (95/03/24)
 	  - Fixed some IPX-related bugs. (Thanks to Tomasz Motylewski
             <motyl@tichy.ch.uj.edu.pl> for the patches to make arcnet work
             with dosemu!)
+            
 	v1.00 (95/02/15)
 	  - Initial non-alpha release.
 	
@@ -85,18 +51,24 @@
 	TO DO:
 	
          - Test in systems with NON-ARCnet network cards, just to see if
-           autoprobe kills anything.  Currently, we do cause some NE2000's to
-           die.
+           autoprobe kills anything.  Currently, we do cause some NE2000's
+           to die.  Autoprobe is also way too slow and verbose, particularly
+           if there ARE no ARCnet cards.
          - What about cards with shared memory that can be "turned off?"
          - NFS mount freezes after several megabytes to SOSS for DOS. 
  	   unmount/remount fixes it.  Is this arcnet-specific?  I don't know.
          - Add support for "old" (RFC1051) protocol arcnet, such as AmiTCP
            and NetBSD.  Work in Tomasz' initial support for this.
-         - How about TCP/IP over netbios?
+         - How about TCP/IP over netbios?  Or vice versa?
          - Some newer ARCnets support promiscuous mode, supposedly. 
            If someone sends me information, I'll try to implement it.
          - Remove excess lock variables that are probably not necessary
            anymore due to the changes in Linux 1.2.9.
+         - Dump Linux 1.2 and properly use the extended 1.3.x skb functions.
+         - Problems forwarding from arc0e to arc0 in 1.3.x? (it may be a
+           general kernel bug, though, since I heard other people complaining
+           about forwarding problems on ethernet cards)
+         - D_SKB doesn't work right on 1.3.x kernels.
 
            
 	Sources:
@@ -115,8 +87,8 @@
 */
 
 static char *version =
- "arcnet.c:v1.91 ALPHA 95/07/02 Avery Pennarun <apenwarr@foxnet.net>\n";
- 
+ "arcnet.c:v2.00 95/09/06 Avery Pennarun <apenwarr@foxnet.net>\n";
+
 /**************************************************************************/
 
 /* Define this if you want to detect network reconfigurations.
@@ -143,7 +115,8 @@ static char *version =
 #define VERIFY_ACK
 
 /* Define this if you want to make it easier to use the "call trace" when
- * a kernel NULL pointer assignment occurs.
+ * a kernel NULL pointer assignment occurs.  Hopefully unnecessary, most of
+ * the time.
  */
 #undef static
 
@@ -151,11 +124,19 @@ static char *version =
  
 
 #include <linux/config.h>
+#include <linux/version.h>
+
 #ifdef MODULE
 #include <linux/module.h>
-#include <linux/version.h>
 #endif /* MODULE */
 
+
+/* are we Linux 1.2.x? */
+#if LINUX_VERSION_CODE < 0x10300
+#define LINUX12
+#endif
+
+
 #include <linux/kernel.h>
 #include <linux/sched.h>
 #include <linux/types.h>
@@ -169,6 +150,7 @@ static char *version =
 #include <linux/timer.h>
 #include <linux/errno.h>
 #include <linux/delay.h>
+#include <linux/if_arp.h>
 
 #include <asm/system.h>
 #include <asm/bitops.h>
@@ -178,7 +160,12 @@ static char *version =
 #include <linux/netdevice.h>
 #include <linux/etherdevice.h>
 #include <linux/skbuff.h>
+
+#ifdef LINUX12
 #include "arp.h"
+#else
+#include <net/arp.h>
+#endif
 
 
 /* new debugging bitflags: each option can be enabled individually.
@@ -189,7 +176,7 @@ static char *version =
  *		and HOSTNAME is your hostname/ip address
  * and then resetting your routes.
  */
-#define D_NORMAL	1	/* D_NORMAL  startup announcement	*/
+#define D_NORMAL	1	/* D_NORMAL  normal operational info	*/
 #define	D_INIT		2	/* D_INIT    show init/probe messages	*/
 #define D_EXTRA		4	/* D_EXTRA   extra information		*/
 /* debug levels past this point give LOTS of output! */
@@ -198,10 +185,11 @@ static char *version =
 #define D_RX		32	/* D_RX	     show rx packets		*/
 #define D_SKB		64	/* D_SKB     dump skb's			*/
 
-#ifndef NET_DEBUG
-#define NET_DEBUG D_NORMAL|D_INIT|D_EXTRA
+#ifndef ARCNET_DEBUG
+/*#define ARCNET_DEBUG D_NORMAL|D_INIT|D_EXTRA*/
+#define ARCNET_DEBUG D_NORMAL|D_INIT
 #endif
-int arcnet_debug = NET_DEBUG;
+int arcnet_debug = ARCNET_DEBUG;
 
 #ifndef HAVE_AUTOIRQ
 /* From auto_irq.c, in ioport.h for later versions. */
@@ -223,11 +211,11 @@ extern struct device *irq2dev_map[16];
 
 /* Some useful multiprotocol macros */
 #define TBUSY lp->adev->tbusy \
-		=lp->wdev->tbusy
+		=lp->edev->tbusy
 #define IF_TBUSY (lp->adev->tbusy \
-		|| lp->wdev->tbusy)
+		|| lp->edev->tbusy)
 #define START lp->adev->start \
-		=lp->wdev->start
+		=lp->edev->start
 
 
 /* The number of low I/O ports used by the ethercard. */
@@ -316,7 +304,7 @@ extern struct device *irq2dev_map[16];
 #define ARC_P_IPX	250		/* 0xFA */
 
 	/* MS LanMan/WfWg protocol */
-#define ARC_P_MS_TCPIP	0xE8
+#define ARC_P_ETHER	0xE8
 
 	/* Unsupported/indirectly supported protocols */
 #define ARC_P_LANSOFT	251		/* 0xFB */
@@ -349,12 +337,11 @@ union ArcPacket
 	 */
 struct ClientData
 {
-	/* data that's NOT part of real packet */
-	u_char	daddr;		/* Destination address - stored here,
-				 *   but WE MUST GET RID OF IT BEFORE SENDING A
-				 *   PACKET!!
-				 */
-	u_char  saddr;		/* Source address - needed for IPX protocol */
+	/* data that's NOT part of real packet - we MUST get rid of it before
+	 * actually sending!!
+	 */
+	u_char  saddr,		/* Source address - needed for IPX */
+		daddr;		/* Destination address */
 	
 	/* data that IS part of real packet */
 	u_char	protocol_id,	/* ARC_P_IP, ARC_P_ARP, etc */
@@ -411,7 +398,7 @@ struct arcnet_local {
 	struct Outgoing outgoing; /* packet currently being sent */
 	
 	struct device *adev;	/* RFC1201 protocol device */
-	struct device *wdev;	/* Windows protocol device */
+	struct device *edev;	/* Ethernet-Encap device */
 };
 
 
@@ -421,7 +408,7 @@ extern int arcnet_probe(struct device *dev);
 static int arcnet_memprobe(struct device *dev,u_char *addr);
 static int arcnet_ioprobe(struct device *dev, short ioaddr);
 #endif
-static int arcnetW_init(struct device *dev);
+static int arcnetE_init(struct device *dev);
 
 static int arcnet_open(struct device *dev);
 static int arcnet_close(struct device *dev);
@@ -433,7 +420,7 @@ static void arcnetA_prepare_tx(struct device *dev,struct ClientData *hdr,
 		short length,char *data);
 static void arcnetA_go_tx(struct device *dev);
 
-static int arcnetW_send_packet(struct sk_buff *skb, struct device *dev);
+static int arcnetE_send_packet(struct sk_buff *skb, struct device *dev);
 
 static void arcnet_interrupt(int irq,struct pt_regs *regs);
 static void arcnet_inthandler(struct device *dev);
@@ -441,15 +428,21 @@ static void arcnet_inthandler(struct device *dev);
 static void arcnet_rx(struct device *dev,int recbuf);
 static void arcnetA_rx(struct device *dev,struct ClientData *arcsoft,
 	int length,u_char saddr, u_char daddr);
-static void arcnetW_rx(struct device *dev,u_char *arcsoft,
+static void arcnetE_rx(struct device *dev,u_char *arcsoft,
 	int length,u_char saddr, u_char daddr);
 
 static struct enet_statistics *arcnet_get_stats(struct device *dev);
 static void set_multicast_list(struct device *dev, int num_addrs, void *addrs);
 
-	/* annoying functions for header/arp/etc building */
-int arcnetA_header(unsigned char *buff,struct device *dev,unsigned short type,
-		void *daddr,void *saddr,unsigned len,struct sk_buff *skb);
+	/* functions for header/arp/etc building */
+#ifdef LINUX12
+int arcnetA_header(unsigned char *buff,struct device *dev,
+		unsigned short type,void *daddr,void *saddr,unsigned len,
+		struct sk_buff *skb);
+#else
+int arcnetA_header(struct sk_buff *skb,struct device *dev,
+		unsigned short type,void *daddr,void *saddr,unsigned len);
+#endif
 int arcnetA_rebuild_header(void *eth,struct device *dev,unsigned long raddr,
 		struct sk_buff *skb);
 unsigned short arcnetA_type_trans(struct sk_buff *skb,struct device *dev);
@@ -511,16 +504,29 @@ arcnet_probe(struct device *dev)
 	int delayval;
 	struct arcnet_local *lp;
 
+#if 0
 	BUGLVL(D_NORMAL)
 	{
 		printk(version);
 		printk("arcnet: ***\n");
-		printk("arcnet: * Read linux/drivers/net/README.arcnet for important release notes!\n");
+		printk("arcnet: * Read README.arcnet for important release notes!\n");
 		printk("arcnet: *\n");
-		printk("arcnet: * This is an ALPHA version!  (Last stable release: v1.02)  E-mail me if\n");
+		printk("arcnet: * This is an ALPHA version!  (Last stable release: v2.00)  E-mail me if\n");
 		printk("arcnet: * you have any questions, comments, or bug reports.\n");
 		printk("arcnet: ***\n");
 	}
+#else
+	BUGLVL(D_INIT)
+	{
+		printk(version);
+		printk("arcnet: ***\n");
+		printk("arcnet: * Read README.arcnet for important release notes!\n");
+		printk("arcnet: *\n");
+		printk("arcnet: * This version should be stable, but please e-mail\n");
+		printk("arcnet: * me if you have any questions or comments.\n");
+		printk("arcnet: ***\n");
+	}
+#endif
 
 	BUGLVL(D_INIT)
 		printk("arcnet: given: base %lXh, IRQ %Xh, shmem %lXh\n",
@@ -647,7 +653,10 @@ arcnet_probe(struct device *dev)
 
 	dev->hard_header=arcnetA_header;
 	dev->rebuild_header=arcnetA_rebuild_header;
+	
+	#ifdef LINUX12
 	dev->type_trans=arcnetA_type_trans;
+	#endif
 
 	return 0;
 }
@@ -882,21 +891,21 @@ int arcnet_reset(struct device *dev)
 	return 0;
 }
 
-static int arcnetW_init(struct device *dev)
+static int arcnetE_init(struct device *dev)
 {
 	struct arcnet_local *lp = (struct arcnet_local *)dev->priv;
 
-	ether_setup(lp->wdev);
+	ether_setup(lp->edev);
 	dev->dev_addr[0]=0;
 	dev->dev_addr[5]=lp->arcnum;
 	dev->mtu=493;	/* MTU is small because of missing packet splitting */
-	lp->wdev->open=NULL;
-	lp->wdev->stop=NULL;
-	lp->wdev->hard_start_xmit=arcnetW_send_packet;
+	lp->edev->open=NULL;
+	lp->edev->stop=NULL;
+	lp->edev->hard_start_xmit=arcnetE_send_packet;
 
 	BUGLVL(D_EXTRA)
-		printk("%s: ARCnet \"Windows\" protocol initialized.\n",
-			lp->wdev->name);
+		printk("%s: ARCnet \"Ethernet-Encap\" protocol initialized.\n",
+			lp->edev->name);
 			
 	return 0;
 }
@@ -949,13 +958,13 @@ arcnet_open(struct device *dev)
 		printk("%s:  ARCnet RFC1201 protocol initialized.\n",
 			lp->adev->name);
 	
-	/* Initialize the Windows protocol driver */
-	lp->wdev=(struct device *)kmalloc(sizeof(struct device),GFP_KERNEL);
-	memcpy(lp->wdev,dev,sizeof(struct device));
-	lp->wdev->name=(char *)kmalloc(10,GFP_KERNEL);
-	sprintf(lp->wdev->name,"%sw",dev->name);
-	lp->wdev->init=arcnetW_init;
-	register_netdev(lp->wdev);
+	/* Initialize the ethernet-encap protocol driver */
+	lp->edev=(struct device *)kmalloc(sizeof(struct device),GFP_KERNEL);
+	memcpy(lp->edev,dev,sizeof(struct device));
+	lp->edev->name=(char *)kmalloc(10,GFP_KERNEL);
+	sprintf(lp->edev->name,"%se",dev->name);
+	lp->edev->init=arcnetE_init;
+	register_netdev(lp->edev);
 
 	/* we're started */
 	START=1;
@@ -975,7 +984,7 @@ arcnet_close(struct device *dev)
 {
 	int ioaddr = dev->base_addr;
 	struct arcnet_local *lp = (struct arcnet_local *)dev->priv;
-
+	
 	TBUSY=1;
 	START=0;
 	
@@ -988,16 +997,17 @@ arcnet_close(struct device *dev)
 	/* do NOT free lp->adev!!  It's static! */
 	lp->adev=NULL;
 	
-	/* free the Windows protocol device */
-	lp->wdev->start=0;
-	lp->wdev->priv=NULL;
-	unregister_netdev(lp->wdev);
-	kfree(lp->wdev->name);
-	kfree(lp->wdev);
-	lp->wdev=NULL;
+	/* free the ethernet-encap protocol device */
+	lp->edev->start=0;
+	lp->edev->priv=NULL;
+	dev_close(lp->edev);
+	unregister_netdev(lp->edev);
+	kfree(lp->edev->name);
+	kfree(lp->edev);
+	lp->edev=NULL;
 
 	/* Update the statistics here. */
-	
+
 #ifdef MODULE
 	MOD_DEC_USE_COUNT;
 #endif
@@ -1404,15 +1414,15 @@ arcnetA_go_tx(struct device *dev)
 }
 
 
-/* Called by the kernel in order to transmit a "Windows" packet.
+/* Called by the kernel in order to transmit an ethernet-type packet.
  */
 static int
-arcnetW_send_packet(struct sk_buff *skb, struct device *dev)
+arcnetE_send_packet(struct sk_buff *skb, struct device *dev)
 {
 	struct arcnet_local *lp = (struct arcnet_local *)dev->priv;
 	
 	BUGLVL(D_DURING)
-		printk("%s: in arcnetW_send_packet (skb=%p)\n",dev->name,skb);
+		printk("%s: in arcnetE_send_packet (skb=%p)\n",dev->name,skb);
 
 	if (IF_TBUSY)
 	{
@@ -1453,8 +1463,8 @@ arcnetW_send_packet(struct sk_buff *skb, struct device *dev)
 		
 		if (length>XMTU)
 		{
-			printk("arcnet: MTU for %s and %s must be <= 493 for Windows protocol.\n",
-				lp->adev->name,lp->wdev->name);
+			printk("arcnet: MTU for %s and %s must be <= 493 for ethernet encap.\n",
+				lp->adev->name,lp->edev->name);
 			printk("arcnet: transmit aborted.\n");
 
 			dev_kfree_skb(skb,FREE_WRITE);
@@ -1496,7 +1506,7 @@ arcnetW_send_packet(struct sk_buff *skb, struct device *dev)
 				arcpacket->hardheader.offset2);
 		
 		arcsoft=&arcpacket->raw[offset];
-		arcsoft[0]=ARC_P_MS_TCPIP;
+		arcsoft[0]=ARC_P_ETHER;
 		arcsoft++;
 		
 		/* copy the packet into ARCnet shmem
@@ -1822,8 +1832,8 @@ arcnet_rx(struct device *dev,int recbuf)
 		arcnetA_rx(dev,(struct ClientData*)arcsoft,
 			length,saddr,daddr);
 		break;
-	case ARC_P_MS_TCPIP:
-		arcnetW_rx(dev,arcsoft,length,saddr,daddr);
+	case ARC_P_ETHER:
+		arcnetE_rx(dev,arcsoft,length,saddr,daddr);
 		break;		
 	default:
 		printk("arcnet: received unknown protocol %d (%Xh)\n",
@@ -1849,6 +1859,11 @@ arcnet_rx(struct device *dev,int recbuf)
        	}
 
 
+	/* clean out the page to make debugging make more sense :) */
+	BUGLVL(D_DURING)
+		memset((void *)arcpacket->raw,0x42,512);
+
+
 	/* If any worth-while packets have been received, a mark_bh(NET_BH)
 	 * has been done by netif_rx and Linux will handle them after we
 	 * return.
@@ -1967,6 +1982,11 @@ arcnetA_rx(struct device *dev,struct ClientData *arcsoft,
                		}
                		printk("\n");
             	}
+
+			#ifndef LINUX12
+			skb->protocol=arcnetA_type_trans(skb,dev);
+			#endif
+
          	netif_rx(skb);
          	lp->stats.rx_packets++;
          }
@@ -2128,6 +2148,12 @@ arcnetA_rx(struct device *dev,struct ClientData *arcsoft,
                			}
                			printk("\n");
             		}
+
+			
+				#ifndef LINUX12
+				skb->protocol=arcnetA_type_trans(skb,dev);
+				#endif
+				
 	         	netif_rx(skb);
         	 	lp->stats.rx_packets++;
         	}
@@ -2135,17 +2161,17 @@ arcnetA_rx(struct device *dev,struct ClientData *arcsoft,
 }
 
 
-/* Packet receiver for non-standard Windows-style packets
+/* Packet receiver for non-standard ethernet-style packets
  */
 static void
-arcnetW_rx(struct device *dev,u_char *arcsoft,
+arcnetE_rx(struct device *dev,u_char *arcsoft,
 	int length,u_char saddr, u_char daddr)
 {
 	struct arcnet_local *lp = (struct arcnet_local *)dev->priv;
 	struct sk_buff *skb;
 	
 	BUGLVL(D_DURING)
-		printk("arcnet: it's a Windows packet (length=%d)\n",
+		printk("arcnet: it's an ethernet-encap packet (length=%d)\n",
 			length);
 
        	skb = alloc_skb(length, GFP_ATOMIC);
@@ -2156,7 +2182,7 @@ arcnetW_rx(struct device *dev,u_char *arcsoft,
        	}
        	
        	skb->len = length;
-       	skb->dev = lp->wdev;
+       	skb->dev = lp->edev;
        	
        	memcpy(skb->data,(u_char *)arcsoft+1,length-1);
 
@@ -2174,6 +2200,10 @@ arcnetW_rx(struct device *dev,u_char *arcsoft,
                 printk("\n");
         }
         
+		#ifndef LINUX12
+		skb->protocol=eth_type_trans(skb,dev);
+		#endif
+        
         netif_rx(skb);
         lp->stats.rx_packets++;
 }
@@ -2223,10 +2253,21 @@ set_multicast_list(struct device *dev, int num_addrs, void *addrs)
  * saddr=NULL	means use device source address (always will anyway)
  * daddr=NULL	means leave destination address (eg unresolved arp)
  */
-int arcnetA_header(unsigned char *buff,struct device *dev,unsigned short type,
-		void *daddr,void *saddr,unsigned len,struct sk_buff *skb)
+#ifdef LINUX12
+int arcnetA_header(unsigned char *buff,struct device *dev,
+		unsigned short type,void *daddr,void *saddr,unsigned len,
+		struct sk_buff *skb)
+#else
+int arcnetA_header(struct sk_buff *skb,struct device *dev,
+		unsigned short type,void *daddr,void *saddr,unsigned len)
+#endif
 {
-	struct ClientData *head = (struct ClientData *)buff;
+	struct ClientData *head = (struct ClientData *)
+#ifdef LINUX12
+		buff;
+#else
+		skb_push(skb,dev->hard_header_len);
+#endif
 /*	struct arcnet_local *lp=(struct arcnet_local *)(dev->priv);*/
 
 	/* set the protocol ID according to RFC-1201 */
@@ -2336,22 +2377,16 @@ unsigned short arcnetA_type_trans(struct sk_buff *skb,struct device *dev)
 {
 	struct ClientData *head = (struct ClientData *) skb->data;
 	struct arcnet_local *lp=(struct arcnet_local *) (dev->priv);
-	/*unsigned char *rawp;*/
+
+#ifndef LINUX12
+	/* Pull off the arcnet header. */
+	skb->mac.raw=skb->data;
+	skb_pull(skb,dev->hard_header_len);
+#endif
 	
 	if (head->daddr==0)
 		skb->pkt_type=PACKET_BROADCAST;
-		
-#if 0 /* code for ethernet with multicast */
-	if(*eth->h_dest&1)
-	{
-		if(memcmp(eth->h_dest,dev->broadcast, ETH_ALEN)==0)
-			skb->pkt_type=PACKET_BROADCAST;
-		else
-			skb->pkt_type=PACKET_MULTICAST;
-	}
-#endif
-	
-	if(dev->flags&IFF_PROMISC)
+	else if (dev->flags&IFF_PROMISC)
 	{
 		/* if we're not sending to ourselves :) */
 		if (head->daddr != dev->dev_addr[0])
@@ -2375,20 +2410,6 @@ unsigned short arcnetA_type_trans(struct sk_buff *skb,struct device *dev)
 		return 0;
 	}
 
-#if 0 /* more ethernet-specific junk */
-	if (ntohs(eth->h_proto) >= 1536)
-		return eth->h_proto;
-		
-	rawp = (unsigned char *)(eth + 1);
-	
-	if (*(unsigned short *)rawp == 0xFFFF)
-		return htons(ETH_P_802_3);
-	if (*(unsigned short *)rawp == 0xAAAA)
-		return htons(ETH_P_SNAP);
-		
-	return htons(ETH_P_802_2);
-#endif
-
 	return htons(ETH_P_IP);
 }
 
@@ -2403,11 +2424,13 @@ unsigned short arcnetA_type_trans(struct sk_buff *skb,struct device *dev)
 
 #ifdef MODULE
 char kernel_version[] = UTS_RELEASE;
+static char devicename[9] = { 0, };
 static struct device thiscard = {
-  "       ",/* if blank, device name inserted by /linux/drivers/net/net_init.c */
+  devicename, /* device name is inserted by linux/drivers/net/net_init.c */
   0, 0, 0, 0,
   0, 0,  /* I/O address, IRQ */
-  0, 0, 0, NULL, arcnet_probe };
+  0, 0, 0, NULL, arcnet_probe
+};
 	
 	
 int io=0x0;	/* <--- EDIT THESE LINES FOR YOUR CONFIGURATION */
